<?php

namespace Drupal\podcast_publisher_analytics\Cron;

use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Cron handler to anonymize download intents.
 */
class RehashCronHandler implements ContainerInjectionInterface {

  const STATE_ID = 'podcast_publisher_analytics.last_cron_run';

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   */
  public function __construct(Connection $database, StateInterface $state) {
    $this->database = $database;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('state')
    );
  }

  /**
   * Anonymize all download intents that are older than 24 hours.
   */
  public function process() {
    // TODO make this variable configurable.
    // 24 hour in seconds.
    $period_length = 86400;
    $now = time();
    $last_cron_run = $this->state->get(static::STATE_ID) ?? 0;

    // Let this cron handler only run every 24h.
    if ($now - $last_cron_run < $period_length) {
      return;
    }

    $this->updateDownloadIntents($now - $period_length);

    // Set last cron run timestamp to now.
    $this->state->set(static::STATE_ID, $now);
  }

  /**
   * Update all unprocessed requester_id values.
   *
   * @param $timestamp
   *   The timestamp that marks the latest intent that should be processed.
   */
  protected function updateDownloadIntents($timestamp) {
    // Get all unprocessed requester ID's
    $statement = $this->database
      ->select('podcast_download_intent', 'pdi')
      ->fields('pdi', ['requester_id'])
      ->condition('pdi.timestamp', $timestamp, '<')
      ->condition('pdi.processed', TRUE, '<>')
      ->execute();

    while ($row = $statement->fetch()) {
      $old_requester_id = $row->requester_id;
      $new_hash = $this->computeNewHash($old_requester_id);

      $this->database->update('podcast_download_intent')
        ->fields([
          'requester_id' => $new_hash,
          'processed' => TRUE,
        ])
        ->condition('timestamp', $timestamp, '<')
        ->condition('processed', TRUE, '<>')
        ->condition('requester_id', $old_requester_id)
        ->execute();
    }
  }

  /**
   * Computes new requester hash that makes it impossible to extract IP address.
   *
   * @param string $old_hash
   *   The old hash.
   *
   * @return string
   *   The new hash.
   */
  protected function computeNewHash($old_hash) {
    $salt = random_bytes(12);
    if (function_exists('openssl_digest')) {
      return openssl_digest($old_hash.$salt, 'sha256');
    }
    elseif (function_exists('crypt')) {
      return crypt($old_hash, $salt);
    }
    return sha1($old_hash.$salt);
  }

}
