<?php

namespace Drupal\podcast_publisher_analytics;

use Drupal\Core\File\FileSystem;
use Drupal\Core\File\FileSystemInterface;
use GeoIp2\Database\Reader;

/**
 * Service class to handle GeoLite DB file.
 */
class GeoLiteService {

  const DOWNLOAD_URL = 'https://cdn.jsdelivr.net/npm/geolite2-city@latest/GeoLite2-City.mmdb.gz';
  const DESTINATION_DIR = 'podcast_publisher/geolite2';
  const DB_FILE_NAME = 'GeoLite2-City.mmdb';

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\File\FileSystem $file_system
   *   The file system service.
   */
  public function __construct(FileSystem $file_system) {
    $this->fileSystem = $file_system;
  }

  /**
   * Gets the GeoLite Reader.
   *
   * @return \GeoIp2\Database\Reader
   *   The Reader.
   * @throws \MaxMind\Db\Reader\InvalidDatabaseException
   */
  public function getReader(): Reader {
    if (!file_exists($this->getDestinationDirectoryUri() . '/' . static::DB_FILE_NAME)) {
      $this->downloadDatabase();
    }
    return new Reader($this->getDestinationDirectoryUri() . '/' . static::DB_FILE_NAME);
  }

  /**
   * Downloads and extracs GeoLite DB from CDN.
   *
   * @return void
   */
  public function downloadDatabase() {
    $download_destination_dir = $this->getDestinationDirectoryUri();
    $download_destination = $download_destination_dir . '/' . static::DB_FILE_NAME;
    $this->fileSystem
      ->prepareDirectory($download_destination_dir, FileSystemInterface::CREATE_DIRECTORY);
    $zip = file_get_contents(static::DOWNLOAD_URL);
    $this->fileSystem->saveData(gzdecode($zip), $download_destination, FileSystem::EXISTS_REPLACE);
  }

  /**
   * Returns URI of the destination directory.
   *
   * @return string
   *   The URI of the destination directory.
   */
  protected function getDestinationDirectoryUri(): string {
    $stream = in_array('private', stream_get_wrappers()) ? 'private' : 'public';
    return $stream . '://' . static::DESTINATION_DIR;
  }

}
