<?php

namespace Drupal\podcast_publisher_analytics\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity reference ID' formatter.
 *
 * @FieldFormatter(
 *   id = "podcast_publisher_download_tracker",
 *   label = @Translation("Download Tracker URL"),
 *   description = @Translation("Create tracking URL for podcast episodes"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class DownloadTrackerUrlFormatter extends EntityReferenceFormatterBase {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->request = $container->get('request_stack')->getCurrentRequest();
    return $instance;
  }


  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    /** @var \Drupal\media\MediaInterface $media */
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $media) {
      /** @var \Drupal\file\FileInterface $file */
      if ($file = $media->field_podcast_audio_file->entity) {
        $url = $this->getTrackingUrl($media, $file);
        $elements[$delta] = [
          '#plain_text' => $url,
          // Create a cache tag entry for the referenced entity. In the case
          // that the referenced entity is deleted, the cache for referring
          // entities must be cleared.
          '#cache' => [
            'tags' => $media->getCacheTags(),
          ],
        ];
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    // This formatter is only available for entity types that reference
    // media items.
    return ($field_definition->getFieldStorageDefinition()->getSetting('target_type') == 'media');
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'source' => 'feed',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['source'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Source'),
      '#description' => $this->t('String to identify the user\'s source, e.g. \'webplayer\' or \'feed\''),
      '#default_value' => $this->getSetting('source'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Source: %source', ['%source' => $this->getSetting('source')]);
    return $summary;
  }

  /**
   * Returns file url
   *
   * @param \Drupal\file\FileInterface $file
   *
   * @return string
   *   The file url.
   */
  protected function getTrackingUrl(MediaInterface $media, FileInterface $file) {
    $file_url = $file->createFileUrl();

    if (UrlHelper::isExternal($file_url)) {
      $file_url_array = parse_url($file_url);
      $file_url_array['query'] .= $file_url_array['query'] ? '&' : '';
      $file_url_array['query'] .= 'external_host=' . urlencode($file_url_array['host']);
      $file_url = $file_url_array['path'] . '?' . $file_url_array['query'];
    }

    return implode('/', [
      '/podcast-publisher',
      $this->getSetting('source'),
      $media->id(),
      $file->id(),
      substr($file_url, 1), // Remove leading slash.
    ]);
  }

}
