<?php

namespace Drupal\podcast_publisher_analytics\Plugin\QueueWorker;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\podcast_publisher_analytics\Entity\DownloadIntent;
use GeoIp2\Database\Reader;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Custom Queue Worker.
 *
 * @QueueWorker(
 *   id = "podcast_download_intent_tracker",
 *   title = @Translation("Custom Queue"),
 *   cron = {"time" = 60}
 * )
 */
class DownloadIntentTracker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * Download intent entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $downloadIntentStorage;

  /**
   * Media storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $mediaStorage;

  /**
   * File storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $fileStorage;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The user agent storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userAgentStorage;

  protected $geoAreaStorage;

  /**
   * Parsed user-agents.json file.
   *
   * @var mixed[]
   */
  protected $userAgents;

  /**
   * The Geo Reader.
   *
   * @var \GeoIp2\Database\Reader
   */
  protected $geoReader;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new class instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->downloadIntentStorage = $entity_type_manager->getStorage('podcast_download_intent');
    $this->mediaStorage = $entity_type_manager->getStorage('media');
    $this->fileStorage = $entity_type_manager->getStorage('file');
    $this->geoAreaStorage = $entity_type_manager->getStorage('podcast_geo_area');
    $this->userAgentStorage = $entity_type_manager->getStorage('podcast_user_agent');
    $this->moduleHandler = $module_handler;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('logger.channel.podcast_publisher_analytics')
    );
  }

  /**
   * @{inheritdoc}
   */
  public function processItem($data) {
    $requester_id = sha1($data['ip'] . $data['user_agent']);
    /** @var \Drupal\podcast_publisher_analytics\Entity\DownloadIntent $download_intent */
    $download_intent = $this->downloadIntentStorage->create([
      'user_agent_string' => $data['user_agent'],
      'source' => $data['source'],
      'requester_id' => $requester_id,
      'timestamp' => $data['timestamp'],
      'processed' => FALSE,
    ]);
    if ($media = $this->mediaStorage->load($data['media_id'])) {
      $download_intent->media->entity = $media;
    }
    if ($file = $this->fileStorage->load($data['file_id'])) {
      $download_intent->file->entity = $file;
    }
    if (isset($data['httprange'])) {
      $download_intent->httprange->value = $data['httprange'];
    }
    if ($user_agent = $this->findOrCreateUserAgentFromUaString($data['user_agent'])) {
      $download_intent->user_agent->entity = $user_agent;
    }
    try {
      $this->processGeoData($download_intent, $data['ip']);
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());
    }
    $download_intent->save();
  }

  /**
   * Returns the parsed user agent data.
   *
   * @return mixed[]
   *   The parsed user agents data.
   */
  protected function getUserAgents() {
    if (!$this->userAgents) {
      $user_agents_file_path = $this->moduleHandler
          ->getModule('podcast_publisher_analytics')
          ->getPath() . '/data/user-agents.json';
      if (is_file($user_agents_file_path)) {
        $this->userAgents = json_decode(file_get_contents($user_agents_file_path));
      }
    }
    return $this->userAgents;
  }

  /**
   * Returns the geolite2 db reader.
   *
   * @return \GeoIp2\Database\Reader
   *   The reader.
   */
  protected function getGeoReader() {
    if (!$this->geoReader) {
      $file_path = $this->moduleHandler
        ->getModule('podcast_publisher_analytics')
        ->getPath() . '/data/GeoLite2-City.mmdb';
      $this->geoReader = new Reader($file_path);
    }
    return $this->geoReader;
  }

  /**
   * Gets or creates the user agent entity that matches the user agent string.
   *
   * @param string $ua_string
   *   The user agent string.
   *
   * @return \Drupal\podcast_publisher_analytics\Entity\UserAgent|null
   *   The user agent or NULL if there is no match.
   */
  protected function findOrCreateUserAgentFromUaString($ua_string) {
    $ua_string = trim($ua_string);

    // Check if a user agent entity is already there or create one.
    $user_agents = $this->userAgentStorage
      ->loadByProperties(['user_agent' => $ua_string]);
    // If it exists we are done.
    /** @var \Drupal\podcast_publisher_analytics\Entity\UserAgent|null $user_agent */
    if ($user_agent = reset($user_agents)) {
      return $user_agent;
    }

    // Create user agent entity from opawg data.
    /** @var \Drupal\podcast_publisher_analytics\Entity\UserAgent $user_agent */
    $user_agent = $this->userAgentStorage->create([
      'user_agent' => $ua_string,
    ]);

    // Find user agent that matches the request's ua string.
    $user_agent_match = NULL;
    foreach ($this->getUserAgents() as $user_agent_data) {
      foreach ($user_agent_data->user_agents as $regex) {
        $compiled_regex = str_replace('/', '\/', $regex);
        if (preg_match("/{$compiled_regex}/", $ua_string) === 1) {
          $user_agent_match = $user_agent_data;
          break(2);
        }
      }
    }
    if ($user_agent_match) {
      $user_agent->client_name->value = $user_agent_match->app;

      if ($user_agent_match->bot) {
        $user_agent->bot->value = TRUE;
      }

      if ($user_agent_match->os) {
        $user_agent->os_name->value = $this->normalizeOS($user_agent_match->os);
      }

      $user_agent->save();
    }
    return $user_agent;
  }

  protected function processGeoData(DownloadIntent $download_intent, string $ip_address) {
    $geo_data = $this->getGeoReader()->city($ip_address);

    $download_intent->lat->value = $geo_data->location->latitude;
    $download_intent->lng->value = $geo_data->location->longitude;

    $geo_area = $this->getArea($geo_data, 'city');
    if ($geo_area) {
      $download_intent->geo_area->entity = $geo_area;
    }
  }

  protected function getParentArea($record, $type) {
    switch ($type) {
      case 'city':
        return $this->getArea($record, 'subdivision');

        break;
      case 'subdivision':
        return $this->getArea($record, 'country');

        break;
      case 'country':
        return $this->getArea($record, 'continent');

        break;
      case 'continent':
        // has no parent
        break;
    }

    return null;
  }

  protected function getArea($record, $type) {
    $subRecord = $record->{$type == 'subdivision' ? 'mostSpecificSubdivision' : $type};

    if (!$subRecord->geonameId) {
      return $this->getParentArea($record, $type);
    }

    if ($areas = $this->geoAreaStorage->loadByProperties(['name' => $subRecord->names['en']])) {
      return reset($areas);
    }

    $area = $this->geoAreaStorage->create([
      'name' => $subRecord->names['en'],
      'area_type' => $type,
    ]);

    if (isset($subRecord->code)) {
      $area->code->value = $subRecord->code;
    } elseif (isset($subRecord->isoCode)) {
      $area->code->value = $subRecord->isoCode;
    }

    if ($area->area_type->value != 'continent') {
      $parent_area = $this->getParentArea($record, $type);
      $area->parent->entity = $parent_area;
    }

    $area->save();

    return $area;
  }

  /**
   * Normalize OS names.
   *
   * @param string $os_name
   *   The OS name.
   *
   * @return string
   *   The normalized OS name.
   */
  private function normalizeOS($os_name) {
    $map = [
      'alexa' => 'Alexa',
      'ios' => 'iOS',
      'android' => 'Android',
      'mac' => 'macOS',
      'macos' => 'macOS',
      'watchos' => 'watchOS',
      'windows' => 'Windows',
      'linux' => 'Linux',
      'sonos' => 'Sonos',
      'homepod_os' => 'HomepodOS',
      'tvos' => 'tvOS',
    ];

    return $map[trim(strtolower($os_name))] ?? $os_name;
  }

}
